uniform mat4 sl_modelView;
uniform mat4 sl_modelViewProj;
uniform mat4 sl_billboard;
uniform vec4 sl_flipTCoordsAndFogDensity;
uniform vec4 sl_fogColor;
uniform vec4 sl_billboardColor;
uniform vec4 sl_fade;
uniform mat4 sl_basis;
uniform vec4 sl_noSpin;

varying float altitude;

void overrideBillboardFog(in vec3 eyePosition, inout vec4 fogColor, inout float fogExponent, inout float fogBlend, inout float fogFade);
void overrideBillboardColor(in vec3 eyePosition, in vec4 modelPos, in vec4 foggedColor, in float cloudFade, in float voxelFade, in float fogFade, inout vec4 finalColor);
vec4 overridePosition(in vec4 position);

void main()
{
    vec3 eyePosition = (sl_modelView * vec4(gl_Vertex.xyz, 1.0)).xyz;
    float fogDistance = length(eyePosition);
    float fogExponent = clamp(fogDistance * sl_flipTCoordsAndFogDensity.y, 0.0, 1.0);
    float fogColorBoost = sl_fade.y;
    float fogFactor = clamp(exp(-(fogExponent * fogExponent)) / fogColorBoost, 0.0, 1.0);
    vec4 preFadedColor;
    bool additiveBlend = sl_flipTCoordsAndFogDensity.w > 0.0;

    vec4 vFogColor = sl_fogColor;

    float fogFade = 1.0 - fogExponent * fogExponent;
    overrideBillboardFog(eyePosition, vFogColor, fogExponent, fogFactor, fogFade);

    if (sl_billboardColor.w > 0.0) {
        preFadedColor.xyz = mix(vFogColor.xyz, sl_billboardColor.xyz, fogFactor);
        preFadedColor.w = sl_billboardColor.w;
    } else {
        vec4 c = gl_Color * vec4(sl_billboardColor.xyz, 1.0);
        preFadedColor = mix(vFogColor, c, fogFactor);
    }

    vec4 texCoord = gl_MultiTexCoord0.xyzw;

    float atlasIndex = gl_Vertex.w;
    float radius = abs(texCoord.x);
    float rotateDirection = sign(texCoord.x);
    float rotateSpeed = abs(texCoord.y);
    float rotateAngle = abs(texCoord.z);
    vec2 dir = vec2(sign(texCoord.y), sign(texCoord.z));
    float fadeRate = abs(texCoord.w);
    bool fadeIn = texCoord.w > 0.0;
    bool fadeOut = texCoord.w < 0.0;

    float voxelFade = 1.0;

    if (sl_fade.z > 0.0) {
        if (fadeIn) {
            voxelFade = fadeRate * (sl_fade.w - sl_fade.z);
        } else if (fadeOut) {
            voxelFade = 1.0 - fadeRate * (sl_fade.w - sl_fade.z);
        }
        voxelFade = clamp(voxelFade, 0.0, 1.0);
    }

    vec4 vertPos = vec4(radius * dir.x, radius * dir.y, 0.0, 1.0);

    vec4 rotatedPos;

    float gradient = sl_noSpin.y;

    if (sl_noSpin.x > 0.0) {
        rotatedPos = vec4(vertPos.xy, 0.0, 1.0);
        float grad = 1.0 - (-dir.y * gradient);
        preFadedColor.xyz *= grad;
    } else {
        float angle = rotateAngle + rotateSpeed * rotateDirection * sl_fade.w;
        angle = mod(angle, 3.14159265 * 2.0);
        float cosLength = cos(angle);
        float sinLength = sin(angle);

        rotatedPos.x = (cosLength * vertPos.x + -sinLength * vertPos.y);
        rotatedPos.y = (sinLength * vertPos.x + cosLength * vertPos.y);
        rotatedPos.z = 0.0;
        rotatedPos.w = 1.0;

        float nposy = sinLength * dir.x + cosLength * dir.y;
        float grad = 1.0 - (-nposy * gradient);
        preFadedColor.xyz *= grad;
    }

    float flattening = sl_noSpin.w;
    rotatedPos.y *= flattening;

    vec4 vert = gl_Vertex * sl_basis;

    bool isAtlas = atlasIndex > 0.0;
    float scale = isAtlas ? 1.2 : 1.0;

    mat4 xlate = mat4(scale, 0.0, 0.0, 0.0,
                      0.0, scale, 0.0, 0.0,
                      0.0, 0.0, scale, 0.0,
                      vert.x, vert.y, vert.z, 1.0);

    mat4 xlatebb = (xlate * sl_billboard);
    mat4 mvb = (sl_modelViewProj * xlatebb);
    gl_Position = overridePosition(mvb * rotatedPos);

    vec4 modelPos = xlate * rotatedPos;
    modelPos = sl_basis * modelPos;
    altitude = modelPos.y;

    vec2 tc = vec2(0.5 * dir.x + 0.5, 0.5 * dir.y * sl_flipTCoordsAndFogDensity.x + 0.5);

    if (isAtlas) {
        tc *= vec2(0.25, 0.25);
        float idx = atlasIndex - 1.0;
        float row = floor(idx / 4.0);
        float col = idx - row * 4.0;
        tc.x += 0.25 * col;
        tc.y += 0.25 * (3.0 - row);
    }

    mat4 mv = (sl_modelView * xlatebb);
    vec3 vEyePosition = (mv * rotatedPos).xyz;

    gl_TexCoord[0] = vec4(tc, 0.0, 1.0);

    float nearFade = 1.0 - clamp(abs(vEyePosition.z) / radius, 0.0, 1.0);
    voxelFade *= 1.0 - (nearFade * nearFade);

    vec4 finalColor;
    if (additiveBlend) {
        finalColor = mix(vec4(0.0, 0.0, 0.0, 0.0), preFadedColor, sl_fade.xxxx * voxelFade * fogFade);
    } else {
        finalColor = vec4(preFadedColor.xyz, sl_fade.x * voxelFade * fogFade * preFadedColor.a);
    }

    overrideBillboardColor(vEyePosition, modelPos, preFadedColor, sl_fade.x, voxelFade, fogFade, finalColor);

    gl_FrontColor = finalColor;
}
